/*=========================================
 DotNetBips.ApplicationBlocks.Data
 Version 1.0
 Date: 19 October, 2002
 Author: Bipin Joshi (webmaster@dotnetbips.com)
=========================================*/
using System;
using System.Data;
using System.Xml;
using System.Collections;


public sealed class DataHelper
{
	private DataHelper() {}

	private static void AttachParameters(IDbCommand command, IDbDataParameter[] commandParameters)
	{
		foreach (IDbDataParameter p in commandParameters)
		{
			if ((p.Direction == ParameterDirection.InputOutput) && (p.Value == null))
			{
				p.Value = DBNull.Value;
			}
				
			command.Parameters.Add(p);
		}
	}

	private static void AssignParameterValues(IDbDataParameter[] commandParameters, object[] parameterValues)
	{
		if ((commandParameters == null) || (parameterValues == null)) 
		{
			return;
		}

		if (commandParameters.Length != parameterValues.Length)
		{
			throw new ArgumentException("Parameter count does not match Parameter Value count.");
		}

		for (int i = 0, j = commandParameters.Length; i < j; i++)
		{
			commandParameters[i].Value = parameterValues[i];
		}
	}

	private static void PrepareCommand(IDbCommand command, IDbConnection connection, IDbTransaction transaction, CommandType commandType, string commandText, IDbDataParameter[] commandParameters)
	{
		if (connection.State != ConnectionState.Open)
		{
			connection.Open();
		}

		command.Connection = connection;

		command.CommandText = commandText;

		if (transaction != null)
		{
			command.Transaction = transaction;
		}

		command.CommandType = commandType;

		if (commandParameters != null)
		{
			AttachParameters(command, commandParameters);
		}

		return;
	}


	public static int ExecuteNonQuery(string connectionString, CommandType commandType, string commandText)
	{
		return ExecuteNonQuery(connectionString, commandType, commandText, (IDbDataParameter[])null);
	}

	public static int ExecuteNonQuery(string connectionString, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		using (IDbConnection cn = DataFactory.CreateConnection())
		{
			cn.ConnectionString=connectionString;
			cn.Open();
			return ExecuteNonQuery(cn, commandType, commandText, commandParameters);
		}
	}

	public static int ExecuteNonQuery(IDbConnection connection, CommandType commandType, string commandText)
	{
		return ExecuteNonQuery(connection, commandType, commandText, (IDbDataParameter[])null);
	}

	public static int ExecuteNonQuery(IDbConnection connection, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{	
		IDbCommand cmd = DataFactory.CreateCommand();
		PrepareCommand(cmd, connection, (IDbTransaction)null, commandType, commandText, commandParameters);
		int retval = cmd.ExecuteNonQuery();
		cmd.Parameters.Clear();
		return retval;
	}

	public static int ExecuteNonQuery(IDbTransaction transaction, CommandType commandType, string commandText)
	{
		return ExecuteNonQuery(transaction, commandType, commandText, (IDbDataParameter[])null);
	}

	public static int ExecuteNonQuery(IDbTransaction transaction, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		IDbCommand cmd = DataFactory.CreateCommand();
		PrepareCommand(cmd, transaction.Connection, transaction, commandType, commandText, commandParameters);
		int retval = cmd.ExecuteNonQuery();
		cmd.Parameters.Clear();
		return retval;
	}


	public static DataSet ExecuteDataset(string connectionString, CommandType commandType, string commandText)
	{
		return ExecuteDataset(connectionString, commandType, commandText, (IDbDataParameter[])null);
	}
	public static DataSet ExecuteDataset(string connectionString, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		using (IDbConnection cn = DataFactory.CreateConnection())
		{
			cn.ConnectionString=connectionString;
			cn.Open();
			return ExecuteDataset(cn, commandType, commandText, commandParameters);
		}
	}

	public static DataSet ExecuteDataset(IDbConnection connection, CommandType commandType, string commandText)
	{
		return ExecuteDataset(connection, commandType, commandText, (IDbDataParameter[])null);
	}
		
	public static DataSet ExecuteDataset(IDbConnection connection, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		IDbCommand cmd = DataFactory.CreateCommand();
		PrepareCommand(cmd, connection, (IDbTransaction)null, commandType, commandText, commandParameters);
		IDbDataAdapter da = DataFactory.CreateDataAdapter();
		da.SelectCommand=cmd;
		DataSet ds = new DataSet();
		da.Fill(ds);
		cmd.Parameters.Clear();
		return ds;						
	}
		
	public static DataSet ExecuteDataset(IDbTransaction transaction, CommandType commandType, string commandText)
	{
		return ExecuteDataset(transaction, commandType, commandText, (IDbDataParameter[])null);
	}

	public static DataSet ExecuteDataset(IDbTransaction transaction, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		IDbCommand cmd = DataFactory.CreateCommand();
		PrepareCommand(cmd, transaction.Connection, transaction, commandType, commandText, commandParameters);
		IDbDataAdapter da = DataFactory.CreateDataAdapter();
		da.SelectCommand=cmd;
		DataSet ds = new DataSet();
		da.Fill(ds);
		cmd.Parameters.Clear();
		return ds;
	}
		
	private enum ConnectionOwnership	
	{
		Internal, 
		External
	}

	private static IDataReader ExecuteReader(IDbConnection connection, IDbTransaction transaction, CommandType commandType, string commandText, IDbDataParameter[] commandParameters, ConnectionOwnership connectionOwnership)
	{	
		IDbCommand cmd = DataFactory.CreateCommand();
		PrepareCommand(cmd, connection, transaction, commandType, commandText, commandParameters);
		IDataReader dr;
		if (connectionOwnership == ConnectionOwnership.External)
		{
			dr = cmd.ExecuteReader();
		}
		else
		{
			dr = cmd.ExecuteReader(CommandBehavior.CloseConnection);
		}
		cmd.Parameters.Clear();
		return dr;
	}

	public static IDataReader ExecuteReader(string connectionString, CommandType commandType, string commandText)
	{
		return ExecuteReader(connectionString, commandType, commandText, (IDbDataParameter[])null);
	}

	public static IDataReader ExecuteReader(string connectionString, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		IDbConnection cn =DataFactory.CreateConnection();
		cn.ConnectionString=connectionString;
		cn.Open();

		try
		{
			return ExecuteReader(cn, null, commandType, commandText, commandParameters,ConnectionOwnership.Internal);
		}
		catch
		{
			cn.Close();
			throw;
		}
	}


	public static IDataReader ExecuteReader(IDbConnection connection, CommandType commandType, string commandText)
	{
		return ExecuteReader(connection, commandType, commandText, (IDbDataParameter[])null);
	}

	public static IDataReader ExecuteReader(IDbConnection connection, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		return ExecuteReader(connection, (IDbTransaction)null, commandType, commandText, commandParameters, ConnectionOwnership.External);
	}


	public static IDataReader ExecuteReader(IDbTransaction transaction, CommandType commandType, string commandText)
	{
		return ExecuteReader(transaction, commandType, commandText, (IDbDataParameter[])null);
	}

	public static IDataReader ExecuteReader(IDbTransaction transaction, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		return ExecuteReader(transaction.Connection, transaction, commandType, commandText, commandParameters, ConnectionOwnership.External);
	}


	public static object ExecuteScalar(string connectionString, CommandType commandType, string commandText)
	{
		return ExecuteScalar(connectionString, commandType, commandText, (IDbDataParameter[])null);
	}

	public static object ExecuteScalar(string connectionString, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		using (IDbConnection cn = DataFactory.CreateConnection())
		{
			cn.ConnectionString=connectionString;
			cn.Open();
			return ExecuteScalar(cn, commandType, commandText, commandParameters);
		}
	}


	public static object ExecuteScalar(IDbConnection connection, CommandType commandType, string commandText)
	{
		return ExecuteScalar(connection, commandType, commandText, (IDbDataParameter[])null);
	}

	public static object ExecuteScalar(IDbConnection connection, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		IDbCommand cmd = DataFactory.CreateCommand();
		PrepareCommand(cmd, connection, (IDbTransaction)null, commandType, commandText, commandParameters);
		object retval = cmd.ExecuteScalar();
		cmd.Parameters.Clear();
		return retval;
	}

	public static object ExecuteScalar(IDbTransaction transaction, CommandType commandType, string commandText)
	{
		return ExecuteScalar(transaction, commandType, commandText, (IDbDataParameter[])null);
	}

	public static object ExecuteScalar(IDbTransaction transaction, CommandType commandType, string commandText, params IDbDataParameter[] commandParameters)
	{
		IDbCommand cmd = DataFactory.CreateCommand();
		PrepareCommand(cmd, transaction.Connection, transaction, commandType, commandText, commandParameters);
		object retval = cmd.ExecuteScalar();
		cmd.Parameters.Clear();
		return retval;
	}

}
